@@ -27,11 +27,10 @@
 ;;; Code:
 
 (require 'cl-lib)
-(require 'haskell-compat)
 (require 'haskell-lexeme)
+(require 'haskell-string)
 (require 'font-lock)
 
-;;;###autoload
 (defgroup haskell-appearance nil
   "Haskell Appearance."
   :group 'haskell)
@@ -86,6 +85,20 @@ be disabled at that position."
   :type '(alist string string)
   :group 'haskell-appearance)
 
+(defcustom haskell-font-lock-keywords
+  ;; `as', `hiding', and `qualified' are part of the import
+  ;; spec syntax, but they are not reserved.
+  ;; `_' can go in here since it has temporary word syntax.
+  '("case" "class" "data" "default" "deriving" "do"
+    "else" "if" "import" "in" "infix" "infixl"
+    "infixr" "instance" "let" "module" "mdo" "newtype" "of"
+    "rec" "pattern" "proc" "signature" "then" "type" "where" "_"
+    "anyclass" "stock" "via")
+  "Identifiers treated as reserved keywords in Haskell."
+  :group 'haskell-appearance
+  :type '(repeat string))
+
+
 (defun haskell-font-lock-dot-is-not-composition (start)
   "Return non-nil if the \".\" at START is not a composition operator.
 This is the case if the \".\" is part of a \"forall <tvar> . <type>\"."
@@ -95,12 +108,23 @@ This is the case if the \".\" is part of a \"forall <tvar> . <type>\"."
                             (line-beginning-position) t)
         (not (or
               (string= " " (string (char-after start)))
+              (null (char-before start))
               (string= " " (string (char-before start))))))))
 
+(defvar haskell-yesod-parse-routes-mode-keywords
+  '(("^\\([^ \t\n]+\\)\\(?:[ \t]+\\([^ \t\n]+\\)\\)?"
+     (1 'font-lock-string-face)
+     (2 'haskell-constructor-face nil lax))))
+
+(define-derived-mode haskell-yesod-parse-routes-mode text-mode "Yesod parseRoutes mode"
+  "Mode for parseRoutes from Yesod."
+  (setq-local font-lock-defaults '(haskell-yesod-parse-routes-mode-keywords t t nil nil)))
+
 (defcustom haskell-font-lock-quasi-quote-modes
   `(("hsx" . xml-mode)
     ("hamlet" . shakespeare-hamlet-mode)
     ("shamlet" . shakespeare-hamlet-mode)
+    ("whamlet" . shakespeare-hamlet-mode)
     ("xmlQQ" . xml-mode)
     ("xml" . xml-mode)
     ("cmd" . shell-mode)
@@ -109,7 +133,10 @@ This is the case if the \".\" is part of a \"forall <tvar> . <type>\"."
     ("jmacroE" . javascript-mode)
     ("r" . ess-mode)
     ("rChan" . ess-mode)
-    ("sql" . sql-mode))
+    ("sql" . sql-mode)
+    ("json" . json-mode)
+    ("aesonQQ" . json-mode)
+    ("parseRoutes" . haskell-yesod-parse-routes-mode))
   "Mapping from quasi quoter token to fontification mode.
 
 If a quasi quote is seen in Haskell code its contents will have
@@ -117,19 +144,16 @@ font faces assigned as if respective mode was enabled."
   :group 'haskell-appearance
   :type '(repeat (cons string symbol)))
 
-;;;###autoload
 (defface haskell-keyword-face
   '((t :inherit font-lock-keyword-face))
   "Face used to highlight Haskell keywords."
   :group 'haskell-appearance)
 
-;;;###autoload
 (defface haskell-type-face
   '((t :inherit font-lock-type-face))
   "Face used to highlight Haskell types"
   :group 'haskell-appearance)
 
-;;;###autoload
 (defface haskell-constructor-face
   '((t :inherit font-lock-type-face))
   "Face used to highlight Haskell constructors."
@@ -137,7 +161,7 @@ font faces assigned as if respective mode was enabled."
 
 ;; This used to be `font-lock-variable-name-face' but it doesn't result in
 ;; a highlighting that's consistent with other modes (it's mostly used
-;; for function defintions).
+;; for function definitions).
 (defface haskell-definition-face
   '((t :inherit font-lock-function-name-face))
   "Face used to highlight Haskell definitions."
@@ -146,19 +170,21 @@ font faces assigned as if respective mode was enabled."
 ;; This is probably just wrong, but it used to use
 ;; `font-lock-function-name-face' with a result that was not consistent with
 ;; other major modes, so I just exchanged with `haskell-definition-face'.
-;;;###autoload
 (defface haskell-operator-face
   '((t :inherit font-lock-variable-name-face))
   "Face used to highlight Haskell operators."
   :group 'haskell-appearance)
 
-;;;###autoload
 (defface haskell-pragma-face
   '((t :inherit font-lock-preprocessor-face))
-  "Face used to highlight Haskell pragmas."
+  "Face used to highlight Haskell pragmas ({-# ... #-})."
+  :group 'haskell-appearance)
+
+(defface haskell-liquid-haskell-annotation-face
+  '((t :inherit haskell-pragma-face))
+  "Face used to highlight LiquidHaskell annotations ({-@ ... @-})."
   :group 'haskell-appearance)
 
-;;;###autoload
 (defface haskell-literate-comment-face
   '((t :inherit font-lock-doc-face))
   "Face with which to fontify literate comments.
@@ -212,16 +238,6 @@ Regexp match data 0 points to the chars."
           ;; no face.  So force evaluation by using `keep'.
           keep)))))
 
-(defconst haskell-font-lock--reverved-ids
-  ;; `as', `hiding', and `qualified' are part of the import
-  ;; spec syntax, but they are not reserved.
-  ;; `_' can go in here since it has temporary word syntax.
-  '("case" "class" "data" "default" "deriving" "do"
-    "else" "if" "import" "in" "infix" "infixl"
-    "infixr" "instance" "let" "module" "mdo" "newtype" "of"
-    "rec" "proc" "then" "type" "where" "_")
-  "Identifiers treated as reserved keywords in Haskell.")
-
 (defun haskell-font-lock--forward-type (&optional ignore)
   "Find where does this type declaration end.
 
@@ -234,7 +250,8 @@ like ::, class, instance, data, newtype, type."
         (token nil)
         ;; we are starting right after ::
         (last-token-was-operator t)
-        (last-token-was-newline nil))
+        (last-token-was-newline nil)
+        (open-parens 0))
     (while cont
       (setq token (haskell-lexeme-looking-at-token 'newline))
 
@@ -245,10 +262,27 @@ like ::, class, instance, data, newtype, type."
         (setq last-token-was-newline (not last-token-was-operator))
         (setq end (match-end 0))
         (goto-char (match-end 0)))
+       ((member (match-string-no-properties 0)
+                    '(")" "]" "}"))
+        (setq open-parens (1- open-parens))
+        (if (< open-parens 0)
+            ;; unmatched closing parenthesis closes type declaration
+            (setq cont nil)
+          (setq end (match-end 0))
+          (goto-char end))
+        (setq last-token-was-newline nil))
+       ((and (member (match-string-no-properties 0)
+                     '("," ";" "|"))
+             (not (member (match-string-no-properties 0) ignore)))
+        (if (equal 0 open-parens)
+            (setq cont nil)
+          (setq last-token-was-operator t)
+          (setq end (match-end 0))
+          (goto-char end))
+        (setq last-token-was-newline nil))
        ((and (or (member (match-string-no-properties 0)
-                         '("<-" "=" "<-" "←"  "," ";"
-                           ")" "]" "}" "|"))
-                 (member (match-string-no-properties 0) haskell-font-lock--reverved-ids))
+                         '("<-" "=" "←"))
+                 (member (match-string-no-properties 0) haskell-font-lock-keywords))
              (not (member (match-string-no-properties 0) ignore)))
         (setq cont nil)
         (setq last-token-was-newline nil))
@@ -256,11 +290,9 @@ like ::, class, instance, data, newtype, type."
                 '("(" "[" "{"))
         (if last-token-was-newline
             (setq cont nil)
-          (goto-char (match-beginning 0))
-          (condition-case err
-                (forward-sexp)
-              (scan-error (goto-char (nth 3 err))))
-          (setq end (point))
+          (setq open-parens (1+ open-parens))
+          (setq end (match-end 0))
+          (goto-char end)
           (setq last-token-was-newline nil)))
        ((member token '(qsymid char string number template-haskell-quote template-haskell-quasi-quote))
         (setq last-token-was-operator (member (haskell-lexeme-classify-by-first-char (char-after (match-beginning 1)))
@@ -281,26 +313,31 @@ like ::, class, instance, data, newtype, type."
     (goto-char end)))
 
 
-(defun haskell-font-lock--put-face-on-type-or-constructor ()
-  "Private function used to put either type or constructor face
-  on an uppercase identifier."
+(defun haskell-font-lock--select-face-on-type-or-constructor ()
+  "Private function used to select either type or constructor face
+on an uppercase identifier."
   (cl-case (haskell-lexeme-classify-by-first-char (char-after (match-beginning 1)))
-    (varid (when (member (match-string 0) haskell-font-lock--reverved-ids)
-             ;; Note: keywords parse as keywords only when not qualified.
-             ;; GHC parses Control.let as a single but illegal lexeme.
-             (when (member (match-string 0) '("class" "instance" "type" "data" "newtype"))
-               (save-excursion
-                 (goto-char (match-end 0))
-                 (save-match-data
-                   (haskell-font-lock--forward-type
-                    (cond
-                     ((member (match-string 0) '("class" "instance"))
-                      '("|"))
-                     ((member (match-string 0) '("type"))
-                      ;; Need to support 'type instance'
-                      '("=" "instance")))))
-                 (add-text-properties (match-end 0) (point) '(font-lock-multiline t haskell-type t))))
-             'haskell-keyword-face))
+    (varid (let ((word (match-string-no-properties 0)))
+             (cond
+              ((member word haskell-font-lock-keywords)
+               ;; Note: keywords parse as keywords only when not qualified.
+               ;; GHC parses Control.let as a single but illegal lexeme.
+               (when (member word '("class" "instance" "type" "data" "newtype"))
+                 (save-excursion
+                   (goto-char (match-end 0))
+                   (save-match-data
+                     (haskell-font-lock--forward-type
+                      (cond
+                       ((member word '("class" "instance"))
+                        '("|"))
+                       ((member word '("type"))
+                        ;; Need to support 'type instance'
+                        '("=" "instance")))))
+                   (add-text-properties (match-end 0) (point) '(font-lock-multiline t haskell-type t))))
+               'haskell-keyword-face)
+              ((member word '("forall"))
+               (when (get-text-property (match-beginning 0) 'haskell-type)
+                 'haskell-keyword-face)))))
     (conid (if (get-text-property (match-beginning 0) 'haskell-type)
                'haskell-type-face
              'haskell-constructor-face))
@@ -321,6 +358,15 @@ like ::, class, instance, data, newtype, type."
                 (add-text-properties (match-end 0) (point) '(font-lock-multiline t haskell-type t)))
               'haskell-operator-face))))
 
+(defun haskell-font-lock--put-face-on-type-or-constructor ()
+  "Private function used to put either type or constructor face
+on an uppercase identifier."
+  (let ((face (haskell-font-lock--select-face-on-type-or-constructor)))
+    (when (and face
+               (not (text-property-not-all (match-beginning 0) (match-end 0) 'face nil)))
+      (put-text-property (match-beginning 0) (match-end 0) 'face face))))
+
+
 (defun haskell-font-lock-keywords ()
   ;; this has to be a function because it depends on global value of
   ;; `haskell-font-lock-symbols'
@@ -329,16 +375,16 @@ like ::, class, instance, data, newtype, type."
          ;; "^>", otherwise a line of code starts with "^".
          (line-prefix "^\\(?:> ?\\)?")
 
-         (varid "\\b[[:lower:]_][[:alnum:]'_]*\\b")
+         (varid "[[:lower:]_][[:alnum:]'_]*")
          ;; We allow ' preceding conids because of DataKinds/PolyKinds
-         (conid "\\b'?[[:upper:]][[:alnum:]'_]*\\b")
+         (conid "'?[[:upper:]][[:alnum:]'_]*")
          (sym "\\s.+")
 
          ;; Top-level declarations
          (topdecl-var
-          (concat line-prefix "\\(" varid "\\(?:\\s-*,\\s-*" varid "\\)*" "\\)\\s-*"
+          (concat line-prefix "\\(" varid "\\(?:\\s-*,\\s-*" varid "\\)*" "\\)"
                   ;; optionally allow for a single newline after identifier
-                  "\\([\n]\\s-+\\)?"
+                  "\\(\\s-+\\|\\s-*[\n]\\s-+\\)"
                   ;; A toplevel declaration can be followed by a definition
                   ;; (=), a type (::) or (∷), a guard, or a pattern which can
                   ;; either be a variable, a constructor, a parenthesized
@@ -357,21 +403,22 @@ like ::, class, instance, data, newtype, type."
     (setq keywords
           `(;; NOTICE the ordering below is significant
             ;;
-            ("^#.*$" 0 'font-lock-preprocessor-face t)
+            ("^#\\(?:[^\\\n]\\|\\\\\\(?:.\\|\n\\|\\'\\)\\)*\\(?:\n\\|\\'\\)" 0 'font-lock-preprocessor-face t)
 
             ,@(haskell-font-lock-symbols-keywords)
 
             ;; Special case for `as', `hiding', `safe' and `qualified', which are
             ;; keywords in import statements but are not otherwise reserved.
-            ("\\<import[ \t]+\\(?:\\(safe\\>\\)[ \t]*\\)?\\(?:\\(qualified\\>\\)[ \t]*\\)?\\(?:\"[^\"]*\"[\t ]*\\)?[^ \t\n()]+[ \t]*\\(?:\\(\\<as\\>\\)[ \t]*[^ \t\n()]+[ \t]*\\)?\\(\\<hiding\\>\\)?"
+            ("\\<import[ \t]+\\(?:\\(safe\\>\\)[ \t]*\\)?\\(?:\\(qualified\\>\\)[ \t]*\\)?\\(?:\"[^\"]*\"[\t ]*\\)?[^ \t\n()]+[ \t]*\\(?:\\(qualified\\>\\)[ \t]*\\)?\\(?:\\(\\<as\\>\\)[ \t]*[^ \t\n()]+[ \t]*\\)?\\(\\<hiding\\>\\)?"
              (1 'haskell-keyword-face nil lax)
              (2 'haskell-keyword-face nil lax)
              (3 'haskell-keyword-face nil lax)
-             (4 'haskell-keyword-face nil lax))
+             (4 'haskell-keyword-face nil lax)
+             (5 'haskell-keyword-face nil lax))
 
             ;; Special case for `foreign import'
             ;; keywords in foreign import statements but are not otherwise reserved.
-            ("\\<\\(foreign\\)[ \t]+\\(import\\)[ \t]+\\(?:\\(ccall\\|stdcall\\|cplusplus\\|jvm\\|dotnet\\)[ \t]+\\)?\\(?:\\(safe\\|unsafe\\|interruptible\\)[ \t]+\\)?"
+            ("\\<\\(foreign\\)[ \t]+\\(import\\)[ \t]+\\(?:\\(ccall\\|capi\\|stdcall\\|cplusplus\\|jvm\\|dotnet\\)[ \t]+\\)?\\(?:\\(safe\\|unsafe\\|interruptible\\)[ \t]+\\)?"
              (1 'haskell-keyword-face nil lax)
              (2 'haskell-keyword-face nil lax)
              (3 'haskell-keyword-face nil lax)
@@ -398,11 +445,11 @@ like ::, class, instance, data, newtype, type."
 
             ;; Toplevel Declarations.
             ;; Place them *before* generic id-and-op highlighting.
-            (,topdecl-var  (1 (unless (member (match-string 1) haskell-font-lock--reverved-ids)
+            (,topdecl-var  (1 (unless (member (match-string 1) haskell-font-lock-keywords)
                                 'haskell-definition-face)))
-            (,topdecl-var2 (2 (unless (member (match-string 2) haskell-font-lock--reverved-ids)
+            (,topdecl-var2 (2 (unless (member (match-string 2) haskell-font-lock-keywords)
                                 'haskell-definition-face)))
-            (,topdecl-bangpat  (1 (unless (member (match-string 1) haskell-font-lock--reverved-ids)
+            (,topdecl-bangpat  (1 (unless (member (match-string 1) haskell-font-lock-keywords)
                                 'haskell-definition-face)))
             (,topdecl-sym  (2 (unless (member (match-string 2) '("\\" "=" "->" "→" "<-" "←" "::" "∷" "," ";" "`"))
                                 'haskell-definition-face)))
@@ -413,10 +460,35 @@ like ::, class, instance, data, newtype, type."
             ("(\\(,*\\|->\\))" 0 'haskell-constructor-face)
             ("\\[\\]" 0 'haskell-constructor-face)
 
-            (,(concat "`" haskell-lexeme-qid-or-qsym "`") 0 'haskell-operator-face)
+            ("`"
+             (0 (if (or (elt (syntax-ppss) 3) (elt (syntax-ppss) 4))
+                    (parse-partial-sexp (point) (point-max) nil nil (syntax-ppss)
+                                        'syntax-table)
+                  (when (save-excursion
+                          (goto-char (match-beginning 0))
+                          (haskell-lexeme-looking-at-backtick))
+                    (goto-char (match-end 0))
+                    (unless (text-property-not-all (match-beginning 1) (match-end 1) 'face nil)
+                      (put-text-property (match-beginning 1) (match-end 1) 'face 'haskell-operator-face))
+                    (unless (text-property-not-all (match-beginning 2) (match-end 2) 'face nil)
+                      (put-text-property (match-beginning 2) (match-end 2) 'face 'haskell-operator-face))
+                    (unless (text-property-not-all (match-beginning 4) (match-end 4) 'face nil)
+                      (put-text-property (match-beginning 4) (match-end 4) 'face 'haskell-operator-face))
+                    (add-text-properties
+                     (match-beginning 0) (match-end 0)
+                     '(font-lock-fontified t fontified t font-lock-multiline t))))))
 
-            (,haskell-lexeme-qid-or-qsym
-             (0 (haskell-font-lock--put-face-on-type-or-constructor)))))
+            (,haskell-lexeme-idsym-first-char
+             (0 (if (or (elt (syntax-ppss) 3) (elt (syntax-ppss) 4))
+                    (parse-partial-sexp (point) (point-max) nil nil (syntax-ppss)
+                                        'syntax-table)
+                  (when (save-excursion
+                          (goto-char (match-beginning 0))
+                          (haskell-lexeme-looking-at-qidsym))
+                    (goto-char (match-end 0))
+                    ;; note that we have to put face ourselves here because font-lock
+                    ;; will use match data from the original matcher
+                    (haskell-font-lock--put-face-on-type-or-constructor)))))))
     keywords))
 
 
@@ -431,8 +503,11 @@ like ::, class, instance, data, newtype, type."
          (concat " haskell-font-lock-fontify-block:" (symbol-name lang-mode)))
       (delete-region (point-min) (point-max))
       (insert string " ") ;; so there's a final property change
-      (unless (eq major-mode lang-mode) (funcall lang-mode))
-      (font-lock-ensure)
+      (cl-letf (((symbol-function 'message)
+                 (lambda (_fmt &rest _args))))
+        ;; silence messages
+        (unless (eq major-mode lang-mode) (funcall lang-mode))
+        (font-lock-ensure))
       (setq pos (point-min))
       (while (setq next (next-single-property-change pos 'face))
         (put-text-property
@@ -458,7 +533,8 @@ like ::, class, instance, data, newtype, type."
                         (goto-char (nth 8 state))
                         (skip-syntax-backward "w._")
                         (buffer-substring-no-properties (point) (nth 8 state))))
-               (lang-mode (cdr (assoc qqname haskell-font-lock-quasi-quote-modes))))
+               (lang-mode (cdr (assoc (haskell-string-drop-qualifier qqname)
+                                      haskell-font-lock-quasi-quote-modes))))
 
           (if (and lang-mode
                    (fboundp lang-mode))
@@ -509,7 +585,7 @@ like ::, class, instance, data, newtype, type."
       (equal (string-to-syntax "<") (syntax-after (point))))
     'haskell-literate-comment-face)
    ;; Detect pragmas. A pragma is enclosed in special comment
-   ;; delimeters {-# .. #-}.
+   ;; delimiters {-# .. #-}.
    ((save-excursion
       (goto-char (nth 8 state))
       (and (looking-at-p "{-#")
@@ -517,6 +593,15 @@ like ::, class, instance, data, newtype, type."
            (goto-char (- (point) 3))
            (looking-at-p "#-}")))
     'haskell-pragma-face)
+   ;; Detect Liquid Haskell annotations enclosed in special comment
+   ;; delimiters {-@ .. @-}.
+   ((save-excursion
+      (goto-char (nth 8 state))
+      (and (looking-at-p "{-@")
+           (forward-comment 1)
+           (goto-char (- (point) 3))
+           (looking-at-p "@-}")))
+    'haskell-liquid-haskell-annotation-face)
    ;; Haddock comment start with either "-- [|^*$]" or "{- ?[|^*$]"
    ;; (note space optional for nested comments and mandatory for
    ;; double dash comments).
@@ -527,31 +612,77 @@ like ::, class, instance, data, newtype, type."
    ;; - there is only whitespace between
    ;;
    ;; We recognize double dash haddock comments by property
-   ;; 'font-lock-doc-face attached to newline. In case of bounded
+   ;; 'font-lock-doc-face attached to newline. In case of {- -}
    ;; comments newline is outside of comment.
    ((save-excursion
       (goto-char (nth 8 state))
       (or (looking-at-p "\\(?:{- ?\\|-- \\)[|^*$]")
-          (and (looking-at-p "--")              ; are we at double dash comment
+          (and (looking-at-p "--")            ; are we at double dash comment
                (forward-line -1)              ; this is nil on first line
                (eq (get-text-property (line-end-position) 'face)
-                   'font-lock-doc-face)              ; is a doc face
+                   'font-lock-doc-face)       ; is a doc face
                (forward-line)
                (skip-syntax-forward "-")      ; see if there is only whitespace
                (eq (point) (nth 8 state)))))  ; we are back in position
-    'font-lock-doc-face)
+    ;; Here we look inside the comment to see if there are substrings
+    ;; worth marking inside we try to emulate as much of haddock as
+    ;; possible.  First we add comment face all over the comment, then
+    ;; we add special features.
+    (let ((beg (nth 8 state))
+          (end (save-excursion
+                 (parse-partial-sexp (point) (point-max) nil nil state
+                                     'syntax-table)
+                 (point)))
+          (emphasis-open-point nil)
+          (strong-open-point nil))
+      (put-text-property beg end 'face 'font-lock-doc-face)
+
+      (when (fboundp 'add-face-text-property)
+        ;; `add-face-text-property' is not defined in Emacs 23
+
+        ;; iterate over chars, take escaped chars unconditionally
+        ;; mark when a construct is opened, close and face it when
+        ;; it is closed
+
+        (save-excursion
+          (while (< (point) end)
+            (if (looking-at "__\\|\\\\.\\|\\\n\\|[/]")
+                (progn
+                  (cond
+                   ((equal (match-string 0) "/")
+                    (if emphasis-open-point
+                        (progn
+                          (add-face-text-property emphasis-open-point (match-end 0)
+                                                  '(:slant italic))
+                          (setq emphasis-open-point nil))
+                      (setq emphasis-open-point (point))))
+                   ((equal (match-string 0) "__")
+                    (if strong-open-point
+                        (progn
+                          (add-face-text-property strong-open-point (match-end 0)
+                                                  '(:weight bold))
+                          (setq strong-open-point nil))
+                      (setq strong-open-point (point))))
+                   (t
+                    ;; this is a backslash escape sequence, skip over it
+                    ))
+                  (goto-char (match-end 0)))
+              ;; skip chars that are not interesting
+              (goto-char (1+ (point)))
+              (skip-chars-forward "^_\\\\/" end))))))
+    nil)
    (t 'font-lock-comment-face)))
 
 (defun haskell-font-lock-defaults-create ()
   "Locally set `font-lock-defaults' for Haskell."
-  (set (make-local-variable 'font-lock-defaults)
-       '((haskell-font-lock-keywords)
-         nil nil nil nil
-         (font-lock-syntactic-face-function
-          . haskell-syntactic-face-function)
-         ;; Get help from font-lock-syntactic-keywords.
-         (parse-sexp-lookup-properties . t)
-         (font-lock-extra-managed-props . (composition)))))
+  (setq-local font-lock-defaults
+              '((haskell-font-lock-keywords)
+                nil nil nil nil
+                (font-lock-syntactic-face-function
+                 . haskell-syntactic-face-function)
+                ;; Get help from font-lock-syntactic-keywords.
+                (parse-sexp-lookup-properties . t)
+                (font-lock-extra-managed-props . (composition)))))
 
 (defun haskell-fontify-as-mode (text mode)
   "Fontify TEXT as MODE, returning the fontified text."
@@ -568,7 +699,7 @@ like ::, class, instance, data, newtype, type."
 (provide 'haskell-font-lock)
 
 ;; Local Variables:
-;; coding: utf-8-unix
+;; coding: utf-8
 ;; tab-width: 8
 ;; End:
 
